"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2024 Olivier Friard

This file is part of BORIS.

  BORIS is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  any later version.

  BORIS is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not see <http://www.gnu.org/licenses/>.

"""

import binascii
import io
import json
import os

from PyQt5.QtCore import QBuffer, QByteArray, QIODevice, QLineF, QPoint, Qt, pyqtSignal
from PyQt5.QtGui import QBrush, QColor, QIcon, QMouseEvent, QPen, QPixmap, QPolygonF
from PyQt5.QtWidgets import (
    QAction,
    QApplication,
    QColorDialog,
    QFileDialog,
    QFrame,
    QGraphicsEllipseItem,
    QGraphicsLineItem,
    QGraphicsPixmapItem,
    QGraphicsPolygonItem,
    QGraphicsScene,
    QGraphicsView,
    QHBoxLayout,
    QInputDialog,
    QLabel,
    QLineEdit,
    QListWidget,
    QMainWindow,
    QMessageBox,
    QPushButton,
    QSizePolicy,
    QSlider,
    QSpacerItem,
    QSplitter,
    QVBoxLayout,
    QWidget,
)

from . import config as cfg
from . import dialog
from . import utilities as util

designColor = QColor(255, 0, 0, 128)  # red opacity: 50%
penWidth = 0
penStyle = Qt.NoPen
selectedBrush = QBrush()
selectedBrush.setStyle(Qt.SolidPattern)
selectedBrush.setColor(QColor(255, 255, 0, 255))


class BehaviorsMapCreatorWindow(QMainWindow):
    signal_add_to_project = pyqtSignal(dict)

    class View(QGraphicsView):
        """
        class for handling mousepress event in QGraphicsView
        """

        mousePress = pyqtSignal(QMouseEvent)

        def mousePressEvent(self, event):
            self.mousePress.emit(event)

        _start = 0
        elList, points = [], []

        def __init__(self, parent):
            QGraphicsView.__init__(self, parent)
            self.setBackgroundBrush(QColor(128, 128, 128))
            self.setScene(QGraphicsScene(self))
            self.scene().update()

    bitmapFileName, mapName, fileName = "", "", ""
    flagNewArea, flag_map_changed = False, False
    polygonsList2 = []
    areaColor = QColor("lime")

    def __init__(self, arg):
        self.codes_list = arg

        super(BehaviorsMapCreatorWindow, self).__init__()

        self.pixmap = QPixmap()
        self.closedPolygon = None
        self.selectedPolygon = None

        self.setWindowTitle("BORIS - Behaviors coding map creator")

        self.newMapAction = QAction(QIcon(), "&New behaviors coding map", self)
        self.newMapAction.setShortcut("Ctrl+N")
        self.newMapAction.setStatusTip("Create a new behaviors coding map")
        self.newMapAction.triggered.connect(self.newMap)

        self.openMapAction = QAction(QIcon(), "&Open a behaviors coding map", self)
        self.openMapAction.setShortcut("Ctrl+O")
        self.openMapAction.setStatusTip("Open a behaviors coding map")
        self.openMapAction.triggered.connect(self.openMap)

        self.saveMapAction = QAction(QIcon(), "&Save the current behaviors coding map", self)
        self.saveMapAction.setShortcut("Ctrl+S")
        self.saveMapAction.setStatusTip("Save the current behaviors coding map")
        self.saveMapAction.setEnabled(False)
        self.saveMapAction.triggered.connect(self.saveMap_clicked)

        self.saveAsMapAction = QAction(QIcon(), "Save the current behaviors coding map as ...", self)
        self.saveAsMapAction.setStatusTip("Save the current behaviors coding map as ...")
        self.saveAsMapAction.setEnabled(False)
        self.saveAsMapAction.triggered.connect(self.saveAsMap_clicked)

        self.mapNameAction = QAction(QIcon(), "&Edit name of behaviors coding map", self)
        self.mapNameAction.setShortcut("Ctrl+M")
        self.mapNameAction.setStatusTip("Edit name of behaviors coding map")
        self.mapNameAction.setEnabled(False)
        self.mapNameAction.triggered.connect(self.mapName_clicked)

        self.addToProject = QAction(QIcon(), "Add coding map to project", self)
        self.addToProject.setStatusTip("Add coding map to project")
        self.addToProject.setEnabled(False)
        self.addToProject.triggered.connect(self.add_to_project)

        self.exitAction = QAction(QIcon(), "&Close", self)
        self.exitAction.setStatusTip("Close")
        self.exitAction.triggered.connect(self.close)

        menubar = self.menuBar()
        fileMenu = menubar.addMenu("&File")
        fileMenu.addAction(self.newMapAction)
        fileMenu.addAction(self.openMapAction)
        fileMenu.addAction(self.saveMapAction)
        fileMenu.addAction(self.saveAsMapAction)
        fileMenu.addSeparator()
        fileMenu.addAction(self.mapNameAction)
        fileMenu.addSeparator()
        fileMenu.addAction(self.addToProject)
        fileMenu.addSeparator()
        fileMenu.addAction(self.exitAction)

        splitter1 = QSplitter(Qt.Vertical)
        """
        splitter1.addWidget(splitter1)
        splitter1.addWidget(bottom)
        """
        """vlayout_list = QVBoxLayout()"""

        self.view = self.View(self)
        self.view.mousePress.connect(self.viewMousePressEvent)
        splitter1.addWidget(self.view)

        vlayout_list = QVBoxLayout()
        vlayout_list.addWidget(QLabel("Defined area"))

        self.area_list = QListWidget(self)
        # self.area_list.setMaximumHeight(120)
        self.area_list.itemClicked.connect(self.area_list_item_click)
        vlayout_list.addWidget(self.area_list)
        w = QWidget()
        w.setLayout(vlayout_list)
        splitter1.addWidget(w)
        splitter1.setSizes([300, 100])
        splitter1.setStretchFactor(2, 8)

        self.btLoad = QPushButton("Load bitmap", self)
        self.btLoad.clicked.connect(self.loadBitmap)
        self.btLoad.setVisible(False)

        hlayout_cmd = QHBoxLayout()

        self.btNewArea = QPushButton("New behavior area", self)
        self.btNewArea.clicked.connect(self.newArea)
        self.btNewArea.setVisible(False)
        hlayout_cmd.addWidget(self.btNewArea)

        self.btSaveArea = QPushButton("Save the behavior area", self)
        self.btSaveArea.clicked.connect(self.saveArea)
        self.btSaveArea.setVisible(False)
        hlayout_cmd.addWidget(self.btSaveArea)

        self.btCancelAreaCreation = QPushButton("Cancel", self)
        self.btCancelAreaCreation.clicked.connect(self.cancelAreaCreation)
        self.btCancelAreaCreation.setVisible(False)
        hlayout_cmd.addWidget(self.btCancelAreaCreation)

        self.btDeleteArea = QPushButton("Delete selected behavior area", self)
        self.btDeleteArea.clicked.connect(self.deleteArea)
        self.btDeleteArea.setVisible(False)
        hlayout_cmd.addWidget(self.btDeleteArea)

        hlayout_area = QHBoxLayout()

        self.lb = QLabel("Behavior")
        self.lb.setVisible(False)
        hlayout_area.addWidget(self.lb)

        self.leAreaCode = QLineEdit(self)
        self.leAreaCode.setReadOnly(True)
        self.leAreaCode.setVisible(False)
        self.leAreaCode.setEnabled(False)
        hlayout_area.addWidget(self.leAreaCode)

        self.btEditAreaCode = QPushButton("Select behavior")
        self.btEditAreaCode.clicked.connect(self.edit_area_code)
        self.btEditAreaCode.setVisible(False)
        hlayout_area.addWidget(self.btEditAreaCode)

        self.btColor = QPushButton(clicked=self.chooseColor)
        self.btColor.setVisible(False)
        self.btColor.setStyleSheet(f"QWidget {{background-color:{self.areaColor.name()}}}")
        hlayout_area.addWidget(self.btColor)

        self.slAlpha = QSlider(Qt.Horizontal)
        self.slAlpha.setRange(20, 100)
        self.slAlpha.setValue(50)
        self.slAlpha.valueChanged.connect(self.slAlpha_changed)
        self.slAlpha.setVisible(False)
        hlayout_area.addWidget(self.slAlpha)

        self.slAlpha_changed(50)

        vlayout_frame = QVBoxLayout()
        vlayout_frame.addLayout(hlayout_cmd)
        vlayout_frame.addLayout(hlayout_area)
        vlayout_frame.addItem(QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding))

        frame = QFrame()
        frame.setFrameStyle(QFrame.Panel | QFrame.Plain)
        frame.setMinimumHeight(120)
        frame.setMaximumHeight(120)

        frame.setLayout(vlayout_frame)

        vlayout = QVBoxLayout()
        """
        vlayout.addWidget(self.view)
        vlayout.addWidget(QLabel("Defined area"))
        vlayout.addWidget(self.area_list)
        """
        vlayout.addWidget(splitter1)
        """vlayout.addLayout(vlayout_view_list)"""

        vlayout.addWidget(frame)
        vlayout.addWidget(self.btLoad)

        main_widget = QWidget(self)
        main_widget.setLayout(vlayout)
        self.setCentralWidget(main_widget)

        self.statusBar().showMessage("")

    def add_to_project(self, item):
        """
        add coding map to project
        """

        map_dict = self.make_coding_map_dict()
        if map_dict["areas"] == {}:
            QMessageBox.critical(
                self,
                cfg.programName,
                (
                    "The behaviors coding map does not contain any behavior area.<br>"
                    "Add some behavior areas before adding the coding map to the project"
                ),
            )

            return

        self.signal_add_to_project.emit(map_dict)

    def area_list_item_click(self, item):
        """
        select the polygon corresponding to the clicked area
        """

        if self.selectedPolygon:
            self.selectedPolygon.setPen(QPen(designColor, penWidth, penStyle, Qt.RoundCap, Qt.RoundJoin))
            self.selectedPolygon = None
            self.selectedPolygonMemBrush = None

        idx = int(item.text().split("#")[1])
        ac, pg = self.polygonsList2[idx]

        self.selectedPolygon = pg

        self.selectedPolygonMemBrush = self.selectedPolygon.brush()

        self.selectedPolygon.setPen(QPen(QColor(255, 0, 0, 255), 2, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))

        self.lb.setVisible(True)
        self.leAreaCode.setText(ac)
        self.leAreaCode.setVisible(True)
        self.btEditAreaCode.setVisible(True)

        self.btDeleteArea.setVisible(True)

        self.areaColor = self.selectedPolygon.brush().color()
        self.btColor.setStyleSheet(f"QWidget {{background-color:{self.selectedPolygon.brush().color().name()}}}")
        self.btColor.setVisible(True)

        self.slAlpha.setValue(int(self.selectedPolygon.brush().color().alpha() / 255 * 100))
        self.slAlpha.setVisible(True)

    def edit_area_code(self):
        """
        select a behavior
        """

        if self.leAreaCode.text() in self.codes_list:
            code_index = self.codes_list.index(self.leAreaCode.text())
        else:
            code_index = 0

        item, ok = QInputDialog.getItem(self, "Select a behavior", "Available behaviors", self.codes_list, code_index, False)
        self.leAreaCode.setText(item)

        if self.selectedPolygon:
            for idx, area in enumerate(self.polygonsList2):
                ac, pg = area
                if pg == self.selectedPolygon:
                    self.polygonsList2[idx] = [self.leAreaCode.text(), pg]
                    break

            self.update_area_list()

    def slAlpha_changed(self, val):
        """
        opacity slider value changed
        """

        self.btColor.setText(f"Opacity: {val} %")
        self.areaColor.setAlpha(int(val / 100 * 255))

        if self.selectedPolygon:
            self.selectedPolygon.setBrush(self.areaColor)
            for idx, area in enumerate(self.polygonsList2):
                ac, pg = area
                if pg == self.selectedPolygon:
                    pg.setBrush(self.areaColor)
                    self.polygonsList2[idx] = [ac, pg]
                    break

        if self.closedPolygon:
            self.closedPolygon.setBrush(self.areaColor)

    def chooseColor(self):
        """
        area color button clicked
        """
        cd = QColorDialog()
        cd.setWindowFlags(Qt.WindowStaysOnTopHint)
        cd.setOptions(QColorDialog.DontUseNativeDialog)

        # col = cd.getColor()
        # if col.isValid():
        if cd.exec_():
            self.areaColor = cd.currentColor()
            self.btColor.setStyleSheet(f"QWidget {{background-color:{self.areaColor.name()}}}")
            self.areaColor.setAlpha(int(self.slAlpha.value() / 100 * 255))

        if self.selectedPolygon:
            self.selectedPolygon.setBrush(self.areaColor)

            for idx, area in enumerate(self.polygonsList2):
                ac, pg = area
                if pg == self.selectedPolygon:
                    pg.setBrush(self.areaColor)
                    self.polygonsList2[idx] = [ac, pg]
                    break

        if self.closedPolygon:
            self.closedPolygon.setBrush(self.areaColor)

    def closeEvent(self, event):
        if self.flag_map_changed:
            response = dialog.MessageDialog(
                "BORIS - Behaviors map creator",
                "What to do about the current unsaved behaviors coding map?",
                [cfg.SAVE, cfg.DISCARD, cfg.CANCEL],
            )

            if response == cfg.SAVE:
                if not self.saveMap_clicked():
                    event.ignore()

            if response == cfg.CANCEL:
                event.ignore()
                return

        self.flag_map_changed = False
        event.accept()

    def viewMousePressEvent(self, event):
        """
        check if area selected with mouse
        """

        def add_polygon():
            # create polygon
            newPolygon = QPolygonF()
            for p in self.view.points:
                newPolygon.append(QPoint(p[0], p[1]))

            # draw polygon a red polygon
            self.closedPolygon = QGraphicsPolygonItem(newPolygon)
            self.closedPolygon.setPen(QPen(designColor, penWidth, penStyle, Qt.RoundCap, Qt.RoundJoin))
            self.closedPolygon.setBrush(self.areaColor)
            self.view.scene().addItem(self.closedPolygon)

        if not self.bitmapFileName:
            return

        test = self.view.mapToScene(event.pos()).toPoint()  # coordinates of clicked point

        if test.x() < 0 or test.y() < 0 or test.x() > self.pixmap.size().width() or test.y() > self.pixmap.size().height():
            return

        if not self.flagNewArea:  # test clicked point for areas
            # reset selected polygon to default pen
            if self.selectedPolygon:
                self.selectedPolygon.setPen(QPen(designColor, penWidth, penStyle, Qt.RoundCap, Qt.RoundJoin))
                self.selectedPolygon = None
                self.selectedPolygonMemBrush = None

            for areaCode, pg in self.polygonsList2:
                if pg.contains(test):
                    self.selectedPolygon = pg

                    self.selectedPolygonMemBrush = self.selectedPolygon.brush()

                    self.selectedPolygon.setPen(QPen(QColor(255, 0, 0, 255), 2, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))

                    self.lb.setVisible(True)
                    self.leAreaCode.setText(areaCode)
                    self.leAreaCode.setVisible(True)
                    # self.leAreaCode.setEnabled(False)
                    self.btEditAreaCode.setVisible(True)

                    self.btDeleteArea.setVisible(True)

                    self.areaColor = self.selectedPolygon.brush().color()
                    self.btColor.setStyleSheet(f"QWidget {{background-color:{self.selectedPolygon.brush().color().name()}}}")
                    self.btColor.setVisible(True)

                    self.slAlpha.setValue(int(self.selectedPolygon.brush().color().alpha() / 255 * 100))
                    self.slAlpha.setVisible(True)

                    break

            if not self.selectedPolygon:
                self.leAreaCode.setVisible(False)
                self.lb.setVisible(False)
                self.btDeleteArea.setVisible(False)
                self.btEditAreaCode.setVisible(False)
                self.btColor.setVisible(False)
                self.slAlpha.setVisible(False)
            return

        # delete last line item
        if (event.buttons() & Qt.RightButton) and not self.closedPolygon:
            if self.view.points:
                self.view.points = self.view.points[0:-1]

            if self.view.points:
                self.view._start = QPoint(self.view.points[-1][0], self.view.points[-1][1])
            else:
                self.view._start = None

            # remove graphical elements
            if self.view.elList:
                self.view.scene().removeItem(self.view.elList[-1])
                self.view.elList = self.view.elList[0:-1]

        # add line item
        if event.buttons() == Qt.LeftButton and not self.closedPolygon:
            if self.view._start:
                end = test

                # test is polygon is crossed
                if len(self.view.points) >= 3:
                    for idx, _ in enumerate(self.view.points[:-2]):
                        if util.intersection(
                            self.view.points[idx],
                            self.view.points[idx + 1],
                            self.view.points[-1],
                            (int(end.x()), int(end.y())),
                        ):
                            QMessageBox.critical(self, "", "The polygon edges can not be intersected")
                            return

                # test if polygon closed (dist min 10 px)
                if abs(end.x() - self.view.points[0][0]) < 10 and abs(end.y() - self.view.points[0][1]) < 10:
                    line = QGraphicsLineItem(QLineF(self.view._start, QPoint(self.view.points[0][0], self.view.points[0][1])))
                    line.setPen(QPen(designColor, penWidth, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))

                    self.view.scene().addItem(line)
                    self.view.elList.append(line)

                    self.statusBar().showMessage("Area completed")

                    # create polygon
                    add_polygon()
                    return

                self.view.points.append((int(end.x()), int(end.y())))

                line = QGraphicsLineItem(QLineF(self.view._start, end))
                line.setPen(QPen(designColor, 2, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
                self.view.scene().addItem(line)
                self.view.elList.append(line)

                self.view._start = test

            else:  # first point
                self.view._start = test

                ellipse = QGraphicsEllipseItem(self.view._start.x(), self.view._start.y(), 3, 3)
                ellipse.setPen(QPen(designColor, 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))

                brush = QBrush()
                brush.setStyle(Qt.SolidPattern)
                brush.setColor(designColor)
                ellipse.setBrush(brush)

                self.view.scene().addItem(ellipse)
                self.view.elList.append(ellipse)

                self.view.points.append((self.view._start.x(), self.view._start.y()))

        # automatically close the polygon
        if event.buttons() == Qt.MiddleButton and not self.closedPolygon:
            # add first point as last point of polygon

            # test is polygon is crossed

            if len(self.view.points) >= 3:
                for idx, _ in enumerate(self.view.points[1:-2]):
                    if util.intersection(
                        self.view.points[idx],
                        self.view.points[idx + 1],
                        self.view.points[-1],
                        self.view.points[0],
                    ):
                        QMessageBox.critical(self, "", "The polygon edges can not be intersected")
                        return

            line = QGraphicsLineItem(
                QLineF(self.view.points[-1][0], self.view.points[-1][1], self.view.points[0][0], self.view.points[0][1])
            )
            line.setPen(QPen(designColor, 2, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
            self.view.scene().addItem(line)
            self.view.elList.append(line)

            self.view.points.append(self.view.points[0])
            add_polygon()
            self.statusBar().showMessage("Area completed")

    def mapName_clicked(self):
        """
        Edit map name
        """

        while True:
            map_name, ok = QInputDialog.getText(
                self, "Behaviors coding map name", "Enter a name for the coding map", QLineEdit.Normal, self.mapName
            )
            if map_name.upper() in self.bcm_list:
                QMessageBox.critical(
                    self,
                    "",
                    (
                        "The name for the new coding map already exists.<br>"
                        f"{', '.join(self.bcm_list)} are already defined.<br>"
                        "To reuse the same name the existing coding map must be deleted (File > Edit project)"
                    ),
                )
            if ok and map_name and map_name.upper() not in self.bcm_list:
                self.mapName = map_name
                self.setWindowTitle(f"{cfg.programName} - Behaviors coding map creator - {self.mapName}")
                break
            if not ok:
                return

    def newMap(self):
        """
        create a new map
        """

        if self.flag_map_changed:
            response = dialog.MessageDialog(
                cfg.programName + " - Behaviors coding map creator",
                "What to do about the current unsaved coding map?",
                [cfg.SAVE, cfg.DISCARD, cfg.CANCEL],
            )

            if response == cfg.SAVE:
                if not self.saveMap_clicked():
                    return

            if response == cfg.CANCEL:
                return

        self.cancelMap()

        while True:
            map_name, ok = QInputDialog.getText(self, "Behaviors coding map name", "Enter a name for the new coding map")
            if map_name.upper() in self.bcm_list:
                QMessageBox.critical(
                    self,
                    "",
                    (
                        "The name for the new coding map already exists.<br>"
                        f"{', '.join(self.bcm_list)} are already defined.<br>"
                        "To reuse the same name the existing coding map must be deleted (File > Edit project)"
                    ),
                )
            if ok and map_name and map_name.upper() not in self.bcm_list:
                self.mapName = map_name
                break
            if not ok:
                return
        """
        if not self.mapName:
            QMessageBox.critical(self, "", "You must define a name for the new coding map")
            return
        """

        self.setWindowTitle(f"{cfg.programName} - Behaviors coding map creator tool - {self.mapName}")

        self.btLoad.setVisible(True)
        self.statusBar().showMessage('Click "Load bitmap" button to select and load a bitmap into the viewer')

    def openMap(self):
        """
        open a coding map from file

        load bitmap from data
        show it in view scene
        """
        if self.flag_map_changed:
            response = dialog.MessageDialog(
                cfg.programName + " - Behaviors coding map creator",
                "What to do about the current unsaved coding map?",
                ["Save", "Discard", "Cancel"],
            )

            if (response == "Save" and not self.saveMap_clicked()) or (response == "Cancel"):
                return

        fn = QFileDialog(self).getOpenFileName(
            self, "Open a behaviors coding map", "", "Behaviors coding map (*.behav_coding_map);;All files (*)"
        )
        fileName = fn[0] if type(fn) is tuple else fn

        if fileName:
            try:
                self.codingMap = json.loads(open(fileName, "r").read())
            except Exception:
                QMessageBox.critical(self, cfg.programName, f"The file {fileName} is not a behaviors coding map.")
                return

            if "coding_map_type" not in self.codingMap or self.codingMap["coding_map_type"] != "BORIS behaviors coding map":
                QMessageBox.critical(self, cfg.programName, f"The file {fileName} is not a BORIS behaviors coding map.")

            self.cancelMap()

            self.mapName = self.codingMap["name"]

            self.setWindowTitle(f"{cfg.programName} - Behaviors coding map creator - {self.mapName}")

            self.bitmapFileName = True

            self.fileName = fileName

            bitmapContent = binascii.a2b_base64(self.codingMap["bitmap"])

            self.pixmap.loadFromData(bitmapContent)

            self.view.setSceneRect(0, 0, self.pixmap.size().width(), self.pixmap.size().height())
            self.view.setMinimumHeight(self.pixmap.size().height())
            # self.view.setMaximumHeight(self.pixmap.size().height())
            pixItem = QGraphicsPixmapItem(self.pixmap)
            pixItem.setPos(0, 0)
            self.view.scene().addItem(pixItem)

            for key in self.codingMap["areas"]:
                areaCode = self.codingMap["areas"][key]["code"]
                points = self.codingMap["areas"][key]["geometry"]

                newPolygon = QPolygonF()
                for p in points:
                    newPolygon.append(QPoint(p[0], p[1]))

                # draw polygon
                """polygon = QGraphicsPolygonItem(None, None) if QT_VERSION_STR[0] == "4" else QGraphicsPolygonItem()"""
                polygon = QGraphicsPolygonItem()
                polygon.setPolygon(newPolygon)
                clr = QColor()
                clr.setRgba(self.codingMap["areas"][key]["color"])
                polygon.setPen(QPen(clr, penWidth, penStyle, Qt.RoundCap, Qt.RoundJoin))
                polygon.setBrush(QBrush(clr, Qt.SolidPattern))

                self.view.scene().addItem(polygon)

                self.polygonsList2.append([areaCode, polygon])

            self.btNewArea.setVisible(True)
            self.btLoad.setVisible(False)

            self.saveMapAction.setEnabled(True)
            self.saveAsMapAction.setEnabled(True)
            self.addToProject.setEnabled(True)
            self.mapNameAction.setEnabled(True)

            self.update_area_list()

        else:
            self.statusBar().showMessage("No file", 5000)

    def make_coding_map_dict(self) -> dict:
        """
        create the coding map dictionary
        the image is encoded in bse64 format
        """
        map_dict = {"coding_map_type": "BORIS behaviors coding map", "name": self.mapName, "areas": {}}

        for ac, pg in self.polygonsList2:
            if not map_dict["areas"]:
                idx = 0
            else:
                idx = max(map_dict["areas"].keys()) + 1

            points = []
            for p in range(pg.polygon().count()):
                points.append([int(pg.polygon().value(p).x()), int(pg.polygon().value(p).y())])

            map_dict["areas"][idx] = {"code": ac, "geometry": points, "color": pg.brush().color().rgba()}

        map_dict["areas"] = json.loads(json.dumps(map_dict["areas"]))

        # Save QPixmap to QByteArray via QBuffer.
        byte_array = QByteArray()
        buffer = QBuffer(byte_array)
        buffer.open(QIODevice.WriteOnly)
        self.pixmap.save(buffer, "PNG")
        string_io = io.BytesIO(byte_array)
        string_io.seek(0)

        # add codified bitmap
        map_dict["bitmap"] = binascii.b2a_base64(string_io.read()).decode("utf-8")

        return map_dict

    def saveMap(self):
        """
        save current coding map in JSON format
        """

        if self.fileName:
            mapDict = self.make_coding_map_dict()

            with open(self.fileName, "w") as outfile:
                outfile.write(json.dumps(mapDict))

            self.flag_map_changed = False

            return True
        else:
            return False

    def saveAsMap_clicked(self):
        filters = "Behaviors coding map (*.behav_coding_map);;All files (*)"

        fn = QFileDialog(self).getSaveFileName(self, "Save behaviors coding map as", "", filters)
        self.fileName = fn[0] if type(fn) is tuple else fn

        if self.fileName:
            if os.path.splitext(self.fileName)[1] != ".behav_coding_map":
                self.fileName += ".behav_coding_map"
            self.saveMap()

    def saveMap_clicked(self):
        if not self.fileName:
            fn = QFileDialog().getSaveFileName(
                self,
                "Save modifiers map",
                self.mapName + ".behav_coding_map",
                "Behaviors coding map (*.behav_coding_map);;All files (*)",
            )
            self.fileName = fn[0] if type(fn) is tuple else fn

            if self.fileName and os.path.splitext(self.fileName)[1] != ".behav_coding_map":
                self.fileName += ".behav_coding_map"

        if self.fileName:
            return self.saveMap()

        return False

    def newArea(self):
        if not self.bitmapFileName:
            QMessageBox.critical(self, cfg.programName, "A bitmap must be loaded before to define areas")
            return

        if self.selectedPolygon:
            self.selectedPolygon.setPen(QPen(designColor, penWidth, penStyle, Qt.RoundCap, Qt.RoundJoin))
            self.selectedPolygon = None

        self.flagNewArea = True
        self.btSaveArea.setVisible(True)
        self.btCancelAreaCreation.setVisible(True)
        self.btNewArea.setVisible(False)
        self.lb.setVisible(True)
        self.leAreaCode.clear()
        self.leAreaCode.setVisible(True)
        # self.leAreaCode.setEnabled(False)
        self.btEditAreaCode.setVisible(True)
        self.btColor.setVisible(True)
        self.slAlpha.setVisible(True)
        self.btDeleteArea.setVisible(False)

        self.statusBar().showMessage(
            ("Click on bitmap to set the vertices of the area with the mouse " "(right click will cancel the last point)")
        )

    def saveArea(self):
        if not self.closedPolygon:
            QMessageBox.critical(
                self,
                cfg.programName,
                ("You must close your area before saving it.\n" "The last vertex must correspond to the first one."),
            )

        if len(self.view.points) < 3:
            QMessageBox.critical(self, cfg.programName, "You must define a closed area")
            return

        # check if no area code
        if not self.leAreaCode.text():
            QMessageBox.critical(self, cfg.programName, "You must define a code for the new behavior area")
            return

        # remove all lines
        for x in self.view.elList:
            self.view.scene().removeItem(x)

        # draw polygon
        self.closedPolygon.setBrush(QBrush(self.areaColor, Qt.SolidPattern))
        # self.polygonsList2[self.leAreaCode.text()] = self.closedPolygon
        self.polygonsList2.append([self.leAreaCode.text(), self.closedPolygon])

        self.closedPolygon, self.flagNewArea = None, None
        self.view._start = 0
        self.view.points, self.view.elList = [], []

        for widget in [
            self.btSaveArea,
            self.btCancelAreaCreation,
            self.lb,
            self.leAreaCode,
            self.btEditAreaCode,
            self.btColor,
            self.slAlpha,
            self.btDeleteArea,
            self.btNewArea,
        ]:
            widget.setVisible(False)

        self.btNewArea.setVisible(True)

        self.leAreaCode.setText("")

        self.flag_map_changed = True
        self.statusBar().showMessage("New area saved", 5000)

        self.update_area_list()

    def cancelAreaCreation(self):
        if self.closedPolygon:
            self.view.scene().removeItem(self.closedPolygon)
            self.closedPolygon = None

        # remove all lines
        for x in self.view.elList:
            self.view.scene().removeItem(x)

        self.view.elList = []

        self.view._start = 0
        self.view.points = []
        self.flagNewArea = False
        self.btCancelAreaCreation.setVisible(False)
        self.btDeleteArea.setVisible(False)
        self.btSaveArea.setVisible(False)
        self.lb.setVisible(False)

        self.btColor.setVisible(False)
        self.slAlpha.setVisible(False)
        self.btNewArea.setVisible(True)

        self.leAreaCode.setVisible(False)
        self.leAreaCode.setText("")

        self.btEditAreaCode.setVisible(False)

    def update_area_list(self):
        self.area_list.clear()
        for idx, area in enumerate(self.polygonsList2):
            ac, pg = area
            self.area_list.addItem(f"{ac} #{idx}")

    def deleteArea(self):
        """
        remove selected area from map
        """

        if self.selectedPolygon:
            self.view.scene().removeItem(self.selectedPolygon)

            to_delete = -1
            for idx, area in enumerate(self.polygonsList2):
                ac, pg = area
                if pg == self.selectedPolygon:
                    to_delete = idx

            if to_delete != -1:
                del self.polygonsList2[to_delete]

            self.flag_map_changed = True

        self.view.elList = []

        self.view._start = 0
        self.view.points = []
        self.flagNewArea = False
        self.btSaveArea.setVisible(False)
        self.lb.setVisible(False)

        self.btColor.setVisible(False)
        self.slAlpha.setVisible(False)
        self.btNewArea.setVisible(True)

        self.leAreaCode.setVisible(False)
        self.leAreaCode.setText("")
        self.btEditAreaCode.setVisible(False)

        self.btDeleteArea.setVisible(False)
        self.statusBar().showMessage("")

        self.update_area_list()

    def cancelMap(self):
        """
        remove current map
        """
        self.flagNewArea = False
        self.polygonsList2 = []
        self.view.scene().clear()
        self.btLoad.setVisible(False)
        self.btDeleteArea.setVisible(False)
        self.btNewArea.setVisible(False)
        self.saveMapAction.setEnabled(False)
        self.saveAsMapAction.setEnabled(False)
        self.addToProject.setEnabled(False)
        self.mapNameAction.setEnabled(False)
        self.statusBar().showMessage("")

        self.flag_map_changed = False

    def loadBitmap(self):
        """
        load bitmap as background for coding map
        resize bitmap to CODING_MAP_RESIZE_W x CODING_MAP_RESIZE_H defined in config.py
        """

        fn = QFileDialog(self).getOpenFileName(self, "Load bitmap", "", "bitmap files (*.png *.jpg);;All files (*)")
        fileName = fn[0] if type(fn) is tuple else fn

        if fileName:
            self.bitmapFileName = fileName

            self.pixmap.load(self.bitmapFileName)

            # scale image
            """
            if (
                self.pixmap.size().width() > cfg.CODING_MAP_RESIZE_W
                or self.pixmap.size().height() > cfg.CODING_MAP_RESIZE_H
            ):
                self.pixmap = self.pixmap.scaled(cfg.CODING_MAP_RESIZE_W, cfg.CODING_MAP_RESIZE_H, Qt.KeepAspectRatio)
                QMessageBox.information(
                    self,
                    cfg.programName,
                    (
                        f"The bitmap was resized to {self.pixmap.size().width()}x{self.pixmap.size().height()} pixels\n"
                        "The original file was not modified"
                    ),
                )
            """

            self.view.setSceneRect(0, 0, self.pixmap.size().width(), self.pixmap.size().height())
            pixitem = QGraphicsPixmapItem(self.pixmap)
            pixitem.setPos(0, 0)
            self.view.scene().addItem(pixitem)

            self.btNewArea.setVisible(True)

            self.btLoad.setVisible(False)
            self.saveMapAction.setEnabled(True)
            self.saveAsMapAction.setEnabled(True)
            self.addToProject.setEnabled(True)
            self.mapNameAction.setEnabled(True)

            self.statusBar().showMessage("""Click "New behavior area" to create a new behavior area""")

            self.flag_map_changed = True


def behaviors_coding_map_creator(self):
    """
    show behaviors coding map creator window
    """

    if not self.project:
        QMessageBox.warning(self, cfg.programName, "No project found", QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
        return

    codes_list = [self.pj[cfg.ETHOGRAM][key][cfg.BEHAVIOR_CODE] for key in self.pj[cfg.ETHOGRAM]]

    self.mapCreatorWindow = BehaviorsMapCreatorWindow(codes_list)
    # behaviors coding map list
    self.mapCreatorWindow.bcm_list = [x["name"].upper() for x in self.pj.get(cfg.BEHAVIORS_CODING_MAP, [])]
    self.mapCreatorWindow.signal_add_to_project.connect(self.behaviors_coding_map_creator_signal_addtoproject)
    self.mapCreatorWindow.move(self.pos())
    self.mapCreatorWindow.resize(cfg.CODING_MAP_RESIZE_W, cfg.CODING_MAP_RESIZE_H)
    self.mapCreatorWindow.show()


if __name__ == "__main__":
    import sys

    app = QApplication(sys.argv)
    window = BehaviorsMapCreatorWindow(["North zone", "East zone", "South zone", "West zone"])
    window.resize(cfg.CODING_MAP_RESIZE_W, cfg.CODING_MAP_RESIZE_H)
    window.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'boris/core.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1509, 813)
        MainWindow.setDockOptions(QtWidgets.QMainWindow.AllowNestedDocks|QtWidgets.QMainWindow.AllowTabbedDocks|QtWidgets.QMainWindow.AnimatedDocks)
        MainWindow.setUnifiedTitleAndToolBarOnMac(False)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.w_logo = QtWidgets.QWidget(self.centralwidget)
        self.w_logo.setObjectName("w_logo")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.w_logo)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.lbLogoBoris = QtWidgets.QLabel(self.w_logo)
        self.lbLogoBoris.setText("")
        self.lbLogoBoris.setScaledContents(False)
        self.lbLogoBoris.setAlignment(QtCore.Qt.AlignCenter)
        self.lbLogoBoris.setObjectName("lbLogoBoris")
        self.verticalLayout_6.addWidget(self.lbLogoBoris)
        self.lbLogoUnito = QtWidgets.QLabel(self.w_logo)
        self.lbLogoUnito.setText("")
        self.lbLogoUnito.setTextFormat(QtCore.Qt.AutoText)
        self.lbLogoUnito.setAlignment(QtCore.Qt.AlignCenter)
        self.lbLogoUnito.setWordWrap(True)
        self.lbLogoUnito.setObjectName("lbLogoUnito")
        self.verticalLayout_6.addWidget(self.lbLogoUnito)
        self.verticalLayout_8.addLayout(self.verticalLayout_6)
        self.verticalLayout_3.addWidget(self.w_logo)
        self.w_obs_info = QtWidgets.QWidget(self.centralwidget)
        self.w_obs_info.setEnabled(True)
        self.w_obs_info.setObjectName("w_obs_info")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.w_obs_info)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.lb_player_status = QtWidgets.QLabel(self.w_obs_info)
        self.lb_player_status.setObjectName("lb_player_status")
        self.verticalLayout_9.addWidget(self.lb_player_status)
        self.lb_current_media_time = QtWidgets.QLabel(self.w_obs_info)
        self.lb_current_media_time.setWordWrap(True)
        self.lb_current_media_time.setObjectName("lb_current_media_time")
        self.verticalLayout_9.addWidget(self.lb_current_media_time)
        self.lb_video_info = QtWidgets.QLabel(self.w_obs_info)
        self.lb_video_info.setObjectName("lb_video_info")
        self.verticalLayout_9.addWidget(self.lb_video_info)
        self.lb_zoom_level = QtWidgets.QLabel(self.w_obs_info)
        self.lb_zoom_level.setObjectName("lb_zoom_level")
        self.verticalLayout_9.addWidget(self.lb_zoom_level)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_9.addItem(spacerItem)
        self.lbFocalSubject = QtWidgets.QLabel(self.w_obs_info)
        self.lbFocalSubject.setWordWrap(True)
        self.lbFocalSubject.setObjectName("lbFocalSubject")
        self.verticalLayout_9.addWidget(self.lbFocalSubject)
        self.lbCurrentStates = QtWidgets.QLabel(self.w_obs_info)
        self.lbCurrentStates.setWordWrap(True)
        self.lbCurrentStates.setObjectName("lbCurrentStates")
        self.verticalLayout_9.addWidget(self.lbCurrentStates)
        self.verticalLayout_10.addLayout(self.verticalLayout_9)
        self.w_live = QtWidgets.QWidget(self.w_obs_info)
        self.w_live.setObjectName("w_live")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.w_live)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.pb_live_obs = QtWidgets.QPushButton(self.w_live)
        self.pb_live_obs.setObjectName("pb_live_obs")
        self.verticalLayout_11.addWidget(self.pb_live_obs)
        self.verticalLayout_10.addWidget(self.w_live)
        self.verticalLayout_3.addWidget(self.w_obs_info)
        self.horizontalLayout.addLayout(self.verticalLayout_3)
        self.horizontalLayout_2.addLayout(self.horizontalLayout)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1509, 22))
        self.menubar.setObjectName("menubar")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuRecent_projects = QtWidgets.QMenu(self.menuFile)
        self.menuRecent_projects.setObjectName("menuRecent_projects")
        self.menuImport_project = QtWidgets.QMenu(self.menuFile)
        self.menuImport_project.setObjectName("menuImport_project")
        self.menuMedia_file_Images_directories = QtWidgets.QMenu(self.menuFile)
        self.menuMedia_file_Images_directories.setObjectName("menuMedia_file_Images_directories")
        self.menuSet_paths_relative_to_project_dir = QtWidgets.QMenu(self.menuMedia_file_Images_directories)
        self.menuSet_paths_relative_to_project_dir.setObjectName("menuSet_paths_relative_to_project_dir")
        self.menuRemove_paths = QtWidgets.QMenu(self.menuMedia_file_Images_directories)
        self.menuRemove_paths.setObjectName("menuRemove_paths")
        self.menuObservations = QtWidgets.QMenu(self.menubar)
        self.menuObservations.setObjectName("menuObservations")
        self.menuCreate_transitions_matrix = QtWidgets.QMenu(self.menuObservations)
        self.menuCreate_transitions_matrix.setObjectName("menuCreate_transitions_matrix")
        self.menuExport_events = QtWidgets.QMenu(self.menuObservations)
        self.menuExport_events.setObjectName("menuExport_events")
        self.menuas_behavioural_sequences = QtWidgets.QMenu(self.menuExport_events)
        self.menuas_behavioural_sequences.setObjectName("menuas_behavioural_sequences")
        self.menuAnalyze = QtWidgets.QMenu(self.menubar)
        self.menuAnalyze.setObjectName("menuAnalyze")
        self.menuPlot_events = QtWidgets.QMenu(self.menuAnalyze)
        self.menuPlot_events.setObjectName("menuPlot_events")
        self.menuInter_rater_reliability = QtWidgets.QMenu(self.menuAnalyze)
        self.menuInter_rater_reliability.setObjectName("menuInter_rater_reliability")
        self.menuSimilarities = QtWidgets.QMenu(self.menuAnalyze)
        self.menuSimilarities.setObjectName("menuSimilarities")
        self.menuPlayback = QtWidgets.QMenu(self.menubar)
        self.menuPlayback.setObjectName("menuPlayback")
        self.menuImage_overlay_on_video_2 = QtWidgets.QMenu(self.menuPlayback)
        self.menuImage_overlay_on_video_2.setObjectName("menuImage_overlay_on_video_2")
        self.menuTools = QtWidgets.QMenu(self.menubar)
        self.menuTools.setObjectName("menuTools")
        self.menuTransitions_flow_diagram = QtWidgets.QMenu(self.menuTools)
        self.menuTransitions_flow_diagram.setObjectName("menuTransitions_flow_diagram")
        self.menu_coding_map_creator = QtWidgets.QMenu(self.menuTools)
        self.menu_coding_map_creator.setObjectName("menu_coding_map_creator")
        self.menuMedia_file = QtWidgets.QMenu(self.menuTools)
        self.menuMedia_file.setObjectName("menuMedia_file")
        MainWindow.setMenuBar(self.menubar)
        self.toolBar = QtWidgets.QToolBar(MainWindow)
        self.toolBar.setEnabled(True)
        self.toolBar.setToolTip("")
        self.toolBar.setObjectName("toolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.dwEthogram = QtWidgets.QDockWidget(MainWindow)
        self.dwEthogram.setFloating(False)
        self.dwEthogram.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dwEthogram.setObjectName("dwEthogram")
        self.dockWidgetContents_3 = QtWidgets.QWidget()
        self.dockWidgetContents_3.setObjectName("dockWidgetContents_3")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.dockWidgetContents_3)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.twEthogram = QtWidgets.QTableWidget(self.dockWidgetContents_3)
        self.twEthogram.setFocusPolicy(QtCore.Qt.NoFocus)
        self.twEthogram.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.twEthogram.setAlternatingRowColors(True)
        self.twEthogram.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.twEthogram.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.twEthogram.setObjectName("twEthogram")
        self.twEthogram.setColumnCount(8)
        self.twEthogram.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.twEthogram.setHorizontalHeaderItem(7, item)
        self.verticalLayout_4.addWidget(self.twEthogram)
        self.verticalLayout_5.addLayout(self.verticalLayout_4)
        self.dwEthogram.setWidget(self.dockWidgetContents_3)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dwEthogram)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setSizeGripEnabled(True)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.dwEvents = QtWidgets.QDockWidget(MainWindow)
        self.dwEvents.setFocusPolicy(QtCore.Qt.NoFocus)
        self.dwEvents.setFloating(False)
        self.dwEvents.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dwEvents.setObjectName("dwEvents")
        self.dockWidgetContents_2 = QtWidgets.QWidget()
        self.dockWidgetContents_2.setObjectName("dockWidgetContents_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents_2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtWidgets.QSplitter(self.dockWidgetContents_2)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName("splitter")
        self.tv_events = QtWidgets.QTableView(self.splitter)
        self.tv_events.setFocusPolicy(QtCore.Qt.NoFocus)
        self.tv_events.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tv_events.setObjectName("tv_events")
        self.verticalLayout.addWidget(self.splitter)
        self.dwEvents.setWidget(self.dockWidgetContents_2)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.dwEvents)
        self.dwSubjects = QtWidgets.QDockWidget(MainWindow)
        self.dwSubjects.setFloating(False)
        self.dwSubjects.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dwSubjects.setObjectName("dwSubjects")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.twSubjects = QtWidgets.QTableWidget(self.dockWidgetContents)
        self.twSubjects.setFocusPolicy(QtCore.Qt.NoFocus)
        self.twSubjects.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.twSubjects.setAlternatingRowColors(True)
        self.twSubjects.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.twSubjects.setObjectName("twSubjects")
        self.twSubjects.setColumnCount(4)
        self.twSubjects.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.twSubjects.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.twSubjects.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.twSubjects.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.twSubjects.setHorizontalHeaderItem(3, item)
        self.verticalLayout_2.addWidget(self.twSubjects)
        self.dwSubjects.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dwSubjects)
        self.actionDocumentation = QtWidgets.QAction(MainWindow)
        self.actionDocumentation.setObjectName("actionDocumentation")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionQuit = QtWidgets.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionPause = QtWidgets.QAction(MainWindow)
        self.actionPause.setObjectName("actionPause")
        self.actionPlay = QtWidgets.QAction(MainWindow)
        self.actionPlay.setObjectName("actionPlay")
        self.actionOpen_video_file = QtWidgets.QAction(MainWindow)
        self.actionOpen_video_file.setObjectName("actionOpen_video_file")
        self.actionReset = QtWidgets.QAction(MainWindow)
        self.actionReset.setObjectName("actionReset")
        self.actionFaster = QtWidgets.QAction(MainWindow)
        self.actionFaster.setEnabled(True)
        self.actionFaster.setObjectName("actionFaster")
        self.actionSlower = QtWidgets.QAction(MainWindow)
        self.actionSlower.setEnabled(True)
        self.actionSlower.setObjectName("actionSlower")
        self.actionJumpForward = QtWidgets.QAction(MainWindow)
        self.actionJumpForward.setObjectName("actionJumpForward")
        self.actionLoad_configuration = QtWidgets.QAction(MainWindow)
        self.actionLoad_configuration.setObjectName("actionLoad_configuration")
        self.actionDelete_selected_events = QtWidgets.QAction(MainWindow)
        self.actionDelete_selected_events.setObjectName("actionDelete_selected_events")
        self.actionDelete_all_events = QtWidgets.QAction(MainWindow)
        self.actionDelete_all_events.setObjectName("actionDelete_all_events")
        self.actionSort_observations = QtWidgets.QAction(MainWindow)
        self.actionSort_observations.setObjectName("actionSort_observations")
        self.actionLoad_observations_file = QtWidgets.QAction(MainWindow)
        self.actionLoad_observations_file.setObjectName("actionLoad_observations_file")
        self.actionSelect_observations = QtWidgets.QAction(MainWindow)
        self.actionSelect_observations.setObjectName("actionSelect_observations")
        self.actionConfigure_states_and_events = QtWidgets.QAction(MainWindow)
        self.actionConfigure_states_and_events.setEnabled(False)
        self.actionConfigure_states_and_events.setObjectName("actionConfigure_states_and_events")
        self.actionEdit_event = QtWidgets.QAction(MainWindow)
        self.actionEdit_event.setObjectName("actionEdit_event")
        self.actionLoad_configuration_file = QtWidgets.QAction(MainWindow)
        self.actionLoad_configuration_file.setObjectName("actionLoad_configuration_file")
        self.actionMedia_file_information = QtWidgets.QAction(MainWindow)
        self.actionMedia_file_information.setObjectName("actionMedia_file_information")
        self.actionStart_live_observation = QtWidgets.QAction(MainWindow)
        self.actionStart_live_observation.setObjectName("actionStart_live_observation")
        self.actionNew_project = QtWidgets.QAction(MainWindow)
        self.actionNew_project.setObjectName("actionNew_project")
        self.actionTime_budget = QtWidgets.QAction(MainWindow)
        self.actionTime_budget.setEnabled(True)
        self.actionTime_budget.setIconVisibleInMenu(True)
        self.actionTime_budget.setObjectName("actionTime_budget")
        self.actionSave_project = QtWidgets.QAction(MainWindow)
        self.actionSave_project.setObjectName("actionSave_project")
        self.actionOpen_project = QtWidgets.QAction(MainWindow)
        self.actionOpen_project.setObjectName("actionOpen_project")
        self.actionSet_offset = QtWidgets.QAction(MainWindow)
        self.actionSet_offset.setObjectName("actionSet_offset")
        self.actionEdit_project = QtWidgets.QAction(MainWindow)
        self.actionEdit_project.setObjectName("actionEdit_project")
        self.actionSave_project_as = QtWidgets.QAction(MainWindow)
        self.actionSave_project_as.setObjectName("actionSave_project_as")
        self.actionPreferences = QtWidgets.QAction(MainWindow)
        self.actionPreferences.setObjectName("actionPreferences")
        self.actionNew_observation = QtWidgets.QAction(MainWindow)
        self.actionNew_observation.setObjectName("actionNew_observation")
        self.actionSave_observation = QtWidgets.QAction(MainWindow)
        self.actionSave_observation.setObjectName("actionSave_observation")
        self.actionClose_observation = QtWidgets.QAction(MainWindow)
        self.actionClose_observation.setObjectName("actionClose_observation")
        self.actionEdit_current_observation = QtWidgets.QAction(MainWindow)
        self.actionEdit_current_observation.setEnabled(False)
        self.actionEdit_current_observation.setObjectName("actionEdit_current_observation")
        self.actionOpen_observation_2 = QtWidgets.QAction(MainWindow)
        self.actionOpen_observation_2.setEnabled(False)
        self.actionOpen_observation_2.setVisible(False)
        self.actionOpen_observation_2.setObjectName("actionOpen_observation_2")
        self.actionAdd_event = QtWidgets.QAction(MainWindow)
        self.actionAdd_event.setObjectName("actionAdd_event")
        self.actionDeselectCurrentSubject = QtWidgets.QAction(MainWindow)
        self.actionDeselectCurrentSubject.setObjectName("actionDeselectCurrentSubject")
        self.actionNext = QtWidgets.QAction(MainWindow)
        self.actionNext.setIconVisibleInMenu(False)
        self.actionNext.setObjectName("actionNext")
        self.actionPrevious = QtWidgets.QAction(MainWindow)
        self.actionPrevious.setObjectName("actionPrevious")
        self.actionJumpTo = QtWidgets.QAction(MainWindow)
        self.actionJumpTo.setEnabled(True)
        self.actionJumpTo.setObjectName("actionJumpTo")
        self.actionJumpBackward = QtWidgets.QAction(MainWindow)
        self.actionJumpBackward.setObjectName("actionJumpBackward")
        self.actionEdit_observation = QtWidgets.QAction(MainWindow)
        self.actionEdit_observation.setEnabled(False)
        self.actionEdit_observation.setVisible(False)
        self.actionEdit_observation.setObjectName("actionEdit_observation")
        self.actionCheckUpdate = QtWidgets.QAction(MainWindow)
        self.actionCheckUpdate.setObjectName("actionCheckUpdate")
        self.actionClose_project = QtWidgets.QAction(MainWindow)
        self.actionClose_project.setObjectName("actionClose_project")
        self.actionObservationsList = QtWidgets.QAction(MainWindow)
        self.actionObservationsList.setObjectName("actionObservationsList")
        self.actionNormalSpeed = QtWidgets.QAction(MainWindow)
        self.actionNormalSpeed.setObjectName("actionNormalSpeed")
        self.actionSnapshot = QtWidgets.QAction(MainWindow)
        self.actionSnapshot.setObjectName("actionSnapshot")
        self.actionFrame_by_frame = QtWidgets.QAction(MainWindow)
        self.actionFrame_by_frame.setCheckable(True)
        self.actionFrame_by_frame.setVisible(False)
        self.actionFrame_by_frame.setObjectName("actionFrame_by_frame")
        self.actionAggregatedEventsTabularFormat = QtWidgets.QAction(MainWindow)
        self.actionAggregatedEventsTabularFormat.setObjectName("actionAggregatedEventsTabularFormat")
        self.actionOpen_observation = QtWidgets.QAction(MainWindow)
        self.actionOpen_observation.setObjectName("actionOpen_observation")
        self.actionAaaa = QtWidgets.QAction(MainWindow)
        self.actionAaaa.setObjectName("actionAaaa")
        self.menuCreate_subtitles_2 = QtWidgets.QAction(MainWindow)
        self.menuCreate_subtitles_2.setObjectName("menuCreate_subtitles_2")
        self.actionUser_guide = QtWidgets.QAction(MainWindow)
        self.actionUser_guide.setObjectName("actionUser_guide")
        self.actionEdit_observation_2 = QtWidgets.QAction(MainWindow)
        self.actionEdit_observation_2.setObjectName("actionEdit_observation_2")
        self.actionCheckStateEvents = QtWidgets.QAction(MainWindow)
        self.actionCheckStateEvents.setObjectName("actionCheckStateEvents")
        self.actionRunEventOutside = QtWidgets.QAction(MainWindow)
        self.actionRunEventOutside.setObjectName("actionRunEventOutside")
        self.actionEdit_selected_events = QtWidgets.QAction(MainWindow)
        self.actionEdit_selected_events.setObjectName("actionEdit_selected_events")
        self.actionShow_spectrogram = QtWidgets.QAction(MainWindow)
        self.actionShow_spectrogram.setObjectName("actionShow_spectrogram")
        self.actionExport_events_as_Praat_TextGrid = QtWidgets.QAction(MainWindow)
        self.actionExport_events_as_Praat_TextGrid.setObjectName("actionExport_events_as_Praat_TextGrid")
        self.actionExtract_events_from_media_files = QtWidgets.QAction(MainWindow)
        self.actionExtract_events_from_media_files.setObjectName("actionExtract_events_from_media_files")
        self.action_geometric_measurements = QtWidgets.QAction(MainWindow)
        self.action_geometric_measurements.setObjectName("action_geometric_measurements")
        self.actionFrame_forward = QtWidgets.QAction(MainWindow)
        self.actionFrame_forward.setObjectName("actionFrame_forward")
        self.actionFrame_backward = QtWidgets.QAction(MainWindow)
        self.actionFrame_backward.setObjectName("actionFrame_backward")
        self.actionFilterBehaviors = QtWidgets.QAction(MainWindow)
        self.actionFilterBehaviors.setObjectName("actionFilterBehaviors")
        self.actionShowAllBehaviors = QtWidgets.QAction(MainWindow)
        self.actionShowAllBehaviors.setObjectName("actionShowAllBehaviors")
        self.actionExport_aggregated_events = QtWidgets.QAction(MainWindow)
        self.actionExport_aggregated_events.setObjectName("actionExport_aggregated_events")
        self.actionCoding_pad = QtWidgets.QAction(MainWindow)
        self.actionCoding_pad.setObjectName("actionCoding_pad")
        self.actionTime_budget_by_behaviors_category = QtWidgets.QAction(MainWindow)
        self.actionTime_budget_by_behaviors_category.setObjectName("actionTime_budget_by_behaviors_category")
        self.actionExport_events_as_SDIS_file = QtWidgets.QAction(MainWindow)
        self.actionExport_events_as_SDIS_file.setObjectName("actionExport_events_as_SDIS_file")
        self.actionRecode_resize_video = QtWidgets.QAction(MainWindow)
        self.actionRecode_resize_video.setObjectName("actionRecode_resize_video")
        self.actionMedia_file_information_2 = QtWidgets.QAction(MainWindow)
        self.actionMedia_file_information_2.setObjectName("actionMedia_file_information_2")
        self.actionCreate_transitions_flow_diagram = QtWidgets.QAction(MainWindow)
        self.actionCreate_transitions_flow_diagram.setObjectName("actionCreate_transitions_flow_diagram")
        self.actionCreate_transitions_flow_diagram_2 = QtWidgets.QAction(MainWindow)
        self.actionCreate_transitions_flow_diagram_2.setObjectName("actionCreate_transitions_flow_diagram_2")
        self.actionAll_transitions = QtWidgets.QAction(MainWindow)
        self.actionAll_transitions.setObjectName("actionAll_transitions")
        self.actionNumber_of_transitions = QtWidgets.QAction(MainWindow)
        self.actionNumber_of_transitions.setObjectName("actionNumber_of_transitions")
        self.actionFrequencies_of_transitions_after_behaviors = QtWidgets.QAction(MainWindow)
        self.actionFrequencies_of_transitions_after_behaviors.setObjectName("actionFrequencies_of_transitions_after_behaviors")
        self.actionFind_replace_events = QtWidgets.QAction(MainWindow)
        self.actionFind_replace_events.setObjectName("actionFind_replace_events")
        self.actionFind_events = QtWidgets.QAction(MainWindow)
        self.actionFind_events.setObjectName("actionFind_events")
        self.actionHow_to_cite_BORIS = QtWidgets.QAction(MainWindow)
        self.actionHow_to_cite_BORIS.setObjectName("actionHow_to_cite_BORIS")
        self.actionZoom1_1_1 = QtWidgets.QAction(MainWindow)
        self.actionZoom1_1_1.setCheckable(True)
        self.actionZoom1_1_1.setObjectName("actionZoom1_1_1")
        self.actionZoom1_2_1 = QtWidgets.QAction(MainWindow)
        self.actionZoom1_2_1.setCheckable(True)
        self.actionZoom1_2_1.setObjectName("actionZoom1_2_1")
        self.actionZoom1_1_2 = QtWidgets.QAction(MainWindow)
        self.actionZoom1_1_2.setCheckable(True)
        self.actionZoom1_1_2.setObjectName("actionZoom1_1_2")
        self.actionZoom1_1_4 = QtWidgets.QAction(MainWindow)
        self.actionZoom1_1_4.setCheckable(True)
        self.actionZoom1_1_4.setObjectName("actionZoom1_1_4")
        self.actionZoom1_fitwindow = QtWidgets.QAction(MainWindow)
        self.actionZoom1_fitwindow.setCheckable(True)
        self.actionZoom1_fitwindow.setObjectName("actionZoom1_fitwindow")
        self.actionZoom2_fitwindow = QtWidgets.QAction(MainWindow)
        self.actionZoom2_fitwindow.setCheckable(True)
        self.actionZoom2_fitwindow.setObjectName("actionZoom2_fitwindow")
        self.actionZoom2_1_4 = QtWidgets.QAction(MainWindow)
        self.actionZoom2_1_4.setCheckable(True)
        self.actionZoom2_1_4.setObjectName("actionZoom2_1_4")
        self.actionZoom2_1_2 = QtWidgets.QAction(MainWindow)
        self.actionZoom2_1_2.setCheckable(True)
        self.actionZoom2_1_2.setObjectName("actionZoom2_1_2")
        self.actionZoom2_1_1 = QtWidgets.QAction(MainWindow)
        self.actionZoom2_1_1.setCheckable(True)
        self.actionZoom2_1_1.setObjectName("actionZoom2_1_1")
        self.actionZoom2_2_1 = QtWidgets.QAction(MainWindow)
        self.actionZoom2_2_1.setCheckable(True)
        self.actionZoom2_2_1.setObjectName("actionZoom2_2_1")
        self.actionFilterSubjects = QtWidgets.QAction(MainWindow)
        self.actionFilterSubjects.setObjectName("actionFilterSubjects")
        self.actionShowAllSubjects = QtWidgets.QAction(MainWindow)
        self.actionShowAllSubjects.setObjectName("actionShowAllSubjects")
        self.actionSend_project = QtWidgets.QAction(MainWindow)
        self.actionSend_project.setObjectName("actionSend_project")
        self.actionViewBehavior = QtWidgets.QAction(MainWindow)
        self.actionViewBehavior.setObjectName("actionViewBehavior")
        self.actionBehaviors_coding_map = QtWidgets.QAction(MainWindow)
        self.actionBehaviors_coding_map.setObjectName("actionBehaviors_coding_map")
        self.action_create_behaviors_coding_map = QtWidgets.QAction(MainWindow)
        self.action_create_behaviors_coding_map.setObjectName("action_create_behaviors_coding_map")
        self.action_create_modifiers_coding_map = QtWidgets.QAction(MainWindow)
        self.action_create_modifiers_coding_map.setObjectName("action_create_modifiers_coding_map")
        self.actionPlot_events1 = QtWidgets.QAction(MainWindow)
        self.actionPlot_events1.setObjectName("actionPlot_events1")
        self.actionPlot_events2 = QtWidgets.QAction(MainWindow)
        self.actionPlot_events2.setObjectName("actionPlot_events2")
        self.actionCohen_s_kappa = QtWidgets.QAction(MainWindow)
        self.actionCohen_s_kappa.setObjectName("actionCohen_s_kappa")
        self.actionTime_budget_report = QtWidgets.QAction(MainWindow)
        self.actionTime_budget_report.setObjectName("actionTime_budget_report")
        self.actionCheckStateEventsSingleObs = QtWidgets.QAction(MainWindow)
        self.actionCheckStateEventsSingleObs.setObjectName("actionCheckStateEventsSingleObs")
        self.actionSubjects_pad = QtWidgets.QAction(MainWindow)
        self.actionSubjects_pad.setObjectName("actionSubjects_pad")
        self.actionShow_data_files = QtWidgets.QAction(MainWindow)
        self.actionShow_data_files.setObjectName("actionShow_data_files")
        self.actionConverters_manager = QtWidgets.QAction(MainWindow)
        self.actionConverters_manager.setObjectName("actionConverters_manager")
        self.actionView_observation = QtWidgets.QAction(MainWindow)
        self.actionView_observation.setObjectName("actionView_observation")
        self.actionRemove_path_from_media_files = QtWidgets.QAction(MainWindow)
        self.actionRemove_path_from_media_files.setVisible(False)
        self.actionRemove_path_from_media_files.setObjectName("actionRemove_path_from_media_files")
        self.actionCheck_project = QtWidgets.QAction(MainWindow)
        self.actionCheck_project.setObjectName("actionCheck_project")
        self.actionBehavior_bar_plot = QtWidgets.QAction(MainWindow)
        self.actionBehavior_bar_plot.setObjectName("actionBehavior_bar_plot")
        self.action_obs_list = QtWidgets.QAction(MainWindow)
        self.action_obs_list.setObjectName("action_obs_list")
        self.actionTest = QtWidgets.QAction(MainWindow)
        self.actionTest.setObjectName("actionTest")
        self.actionTest_stb2 = QtWidgets.QAction(MainWindow)
        self.actionTest_stb2.setObjectName("actionTest_stb2")
        self.actionClose_unpaired_events = QtWidgets.QAction(MainWindow)
        self.actionClose_unpaired_events.setObjectName("actionClose_unpaired_events")
        self.actionRotate_video = QtWidgets.QAction(MainWindow)
        self.actionRotate_video.setObjectName("actionRotate_video")
        self.actionExport_observations_list = QtWidgets.QAction(MainWindow)
        self.actionExport_observations_list.setObjectName("actionExport_observations_list")
        self.actionExportEvents_2 = QtWidgets.QAction(MainWindow)
        self.actionExportEvents_2.setObjectName("actionExportEvents_2")
        self.actionJWatcher = QtWidgets.QAction(MainWindow)
        self.actionJWatcher.setObjectName("actionJWatcher")
        self.actionCloseObs = QtWidgets.QAction(MainWindow)
        self.actionCloseObs.setEnabled(True)
        self.actionCloseObs.setObjectName("actionCloseObs")
        self.actionFilter_events = QtWidgets.QAction(MainWindow)
        self.actionFilter_events.setObjectName("actionFilter_events")
        self.actionShow_all_events = QtWidgets.QAction(MainWindow)
        self.actionShow_all_events.setObjectName("actionShow_all_events")
        self.actionEdit_event_time = QtWidgets.QAction(MainWindow)
        self.actionEdit_event_time.setObjectName("actionEdit_event_time")
        self.actionCopy_events = QtWidgets.QAction(MainWindow)
        self.actionCopy_events.setObjectName("actionCopy_events")
        self.actionPaste_events = QtWidgets.QAction(MainWindow)
        self.actionPaste_events.setObjectName("actionPaste_events")
        self.actionNoldus_Observer_template = QtWidgets.QAction(MainWindow)
        self.actionNoldus_Observer_template.setObjectName("actionNoldus_Observer_template")
        self.actionExplore_project = QtWidgets.QAction(MainWindow)
        self.actionExplore_project.setObjectName("actionExplore_project")
        self.actionShow_the_sound_waveform = QtWidgets.QAction(MainWindow)
        self.actionShow_the_sound_waveform.setObjectName("actionShow_the_sound_waveform")
        self.actionNeedleman_Wunsch = QtWidgets.QAction(MainWindow)
        self.actionNeedleman_Wunsch.setObjectName("actionNeedleman_Wunsch")
        self.actionExtract_frames_from_media_files = QtWidgets.QAction(MainWindow)
        self.actionExtract_frames_from_media_files.setObjectName("actionExtract_frames_from_media_files")
        self.action_behavior_binary_table = QtWidgets.QAction(MainWindow)
        self.action_behavior_binary_table.setEnabled(True)
        self.action_behavior_binary_table.setObjectName("action_behavior_binary_table")
        self.actionPlot_current_observation = QtWidgets.QAction(MainWindow)
        self.actionPlot_current_observation.setObjectName("actionPlot_current_observation")
        self.actionFind_in_current_obs = QtWidgets.QAction(MainWindow)
        self.actionFind_in_current_obs.setObjectName("actionFind_in_current_obs")
        self.action_advanced_event_filtering = QtWidgets.QAction(MainWindow)
        self.action_advanced_event_filtering.setObjectName("action_advanced_event_filtering")
        self.action_block_dockwidgets = QtWidgets.QAction(MainWindow)
        self.action_block_dockwidgets.setCheckable(True)
        self.action_block_dockwidgets.setObjectName("action_block_dockwidgets")
        self.actionseparated_subjects = QtWidgets.QAction(MainWindow)
        self.actionseparated_subjects.setObjectName("actionseparated_subjects")
        self.actiongrouped_subjects = QtWidgets.QAction(MainWindow)
        self.actiongrouped_subjects.setObjectName("actiongrouped_subjects")
        self.actionCurrent_Time_Budget = QtWidgets.QAction(MainWindow)
        self.actionCurrent_Time_Budget.setObjectName("actionCurrent_Time_Budget")
        self.actionSynthetic_binned_time_budget = QtWidgets.QAction(MainWindow)
        self.actionSynthetic_binned_time_budget.setObjectName("actionSynthetic_binned_time_budget")
        self.actionZoom_level = QtWidgets.QAction(MainWindow)
        self.actionZoom_level.setObjectName("actionZoom_level")
        self.actionDisplay_subtitles = QtWidgets.QAction(MainWindow)
        self.actionDisplay_subtitles.setObjectName("actionDisplay_subtitles")
        self.actionAdd_image_overlay_on_video = QtWidgets.QAction(MainWindow)
        self.actionAdd_image_overlay_on_video.setObjectName("actionAdd_image_overlay_on_video")
        self.actionRemove_image_overlay = QtWidgets.QAction(MainWindow)
        self.actionRemove_image_overlay.setObjectName("actionRemove_image_overlay")
        self.actionPlot_events_in_real_time = QtWidgets.QAction(MainWindow)
        self.actionPlot_events_in_real_time.setObjectName("actionPlot_events_in_real_time")
        self.actionVideo_equalizer = QtWidgets.QAction(MainWindow)
        self.actionVideo_equalizer.setObjectName("actionVideo_equalizer")
        self.actionRemove_observations = QtWidgets.QAction(MainWindow)
        self.actionRemove_observations.setObjectName("actionRemove_observations")
        self.action_latency = QtWidgets.QAction(MainWindow)
        self.action_latency.setObjectName("action_latency")
        self.actionUndo = QtWidgets.QAction(MainWindow)
        self.actionUndo.setObjectName("actionUndo")
        self.actionaaa = QtWidgets.QAction(MainWindow)
        self.actionaaa.setObjectName("actionaaa")
        self.action_media_file_and_images_directories_relative_path = QtWidgets.QAction(MainWindow)
        self.action_media_file_and_images_directories_relative_path.setObjectName("action_media_file_and_images_directories_relative_path")
        self.action_data_files_relative_path = QtWidgets.QAction(MainWindow)
        self.action_data_files_relative_path.setObjectName("action_data_files_relative_path")
        self.action_remove_media_files_and_images_directories_path = QtWidgets.QAction(MainWindow)
        self.action_remove_media_files_and_images_directories_path.setObjectName("action_remove_media_files_and_images_directories_path")
        self.action_remove_data_files_path = QtWidgets.QAction(MainWindow)
        self.action_remove_data_files_path.setObjectName("action_remove_data_files_path")
        self.actionPlot_current_time_budget = QtWidgets.QAction(MainWindow)
        self.actionPlot_current_time_budget.setObjectName("actionPlot_current_time_budget")
        self.actionMerge_media_files = QtWidgets.QAction(MainWindow)
        self.actionMerge_media_files.setObjectName("actionMerge_media_files")
        self.actionExport_project = QtWidgets.QAction(MainWindow)
        self.actionExport_project.setObjectName("actionExport_project")
        self.actionConfigure_twEvents_columns = QtWidgets.QAction(MainWindow)
        self.actionConfigure_twEvents_columns.setObjectName("actionConfigure_twEvents_columns")
        self.actionRotate_current_video = QtWidgets.QAction(MainWindow)
        self.actionRotate_current_video.setObjectName("actionRotate_current_video")
        self.action_deinterlace = QtWidgets.QAction(MainWindow)
        self.action_deinterlace.setCheckable(True)
        self.action_deinterlace.setObjectName("action_deinterlace")
        self.action_cooccurence = QtWidgets.QAction(MainWindow)
        self.action_cooccurence.setObjectName("action_cooccurence")
        self.action_create_observations = QtWidgets.QAction(MainWindow)
        self.action_create_observations.setObjectName("action_create_observations")
        self.actionConfigure_tvevents_columns = QtWidgets.QAction(MainWindow)
        self.actionConfigure_tvevents_columns.setObjectName("actionConfigure_tvevents_columns")
        self.actionAdd_frame_indexes = QtWidgets.QAction(MainWindow)
        self.actionAdd_frame_indexes.setObjectName("actionAdd_frame_indexes")
        self.menuHelp.addAction(self.actionUser_guide)
        self.menuHelp.addAction(self.actionCheckUpdate)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.actionAbout)
        self.menuRecent_projects.addSeparator()
        self.menuImport_project.addAction(self.actionNoldus_Observer_template)
        self.menuSet_paths_relative_to_project_dir.addAction(self.action_media_file_and_images_directories_relative_path)
        self.menuSet_paths_relative_to_project_dir.addAction(self.action_data_files_relative_path)
        self.menuRemove_paths.addAction(self.action_remove_media_files_and_images_directories_path)
        self.menuRemove_paths.addAction(self.action_remove_data_files_path)
        self.menuMedia_file_Images_directories.addAction(self.actionRemove_path_from_media_files)
        self.menuMedia_file_Images_directories.addAction(self.menuSet_paths_relative_to_project_dir.menuAction())
        self.menuMedia_file_Images_directories.addAction(self.menuRemove_paths.menuAction())
        self.menuFile.addAction(self.actionNew_project)
        self.menuFile.addAction(self.actionOpen_project)
        self.menuFile.addAction(self.menuRecent_projects.menuAction())
        self.menuFile.addAction(self.menuImport_project.menuAction())
        self.menuFile.addAction(self.actionEdit_project)
        self.menuFile.addAction(self.actionSave_project)
        self.menuFile.addAction(self.actionSave_project_as)
        self.menuFile.addAction(self.actionExport_project)
        self.menuFile.addAction(self.actionClose_project)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionCheck_project)
        self.menuFile.addAction(self.menuMedia_file_Images_directories.menuAction())
        self.menuFile.addAction(self.actionSend_project)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionPreferences)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menuCreate_transitions_matrix.addAction(self.actionAll_transitions)
        self.menuCreate_transitions_matrix.addAction(self.actionFrequencies_of_transitions_after_behaviors)
        self.menuCreate_transitions_matrix.addAction(self.actionNumber_of_transitions)
        self.menuas_behavioural_sequences.addAction(self.actionseparated_subjects)
        self.menuas_behavioural_sequences.addAction(self.actiongrouped_subjects)
        self.menuExport_events.addAction(self.actionExportEvents_2)
        self.menuExport_events.addAction(self.actionExport_aggregated_events)
        self.menuExport_events.addAction(self.menuas_behavioural_sequences.menuAction())
        self.menuExport_events.addAction(self.actionExport_events_as_Praat_TextGrid)
        self.menuExport_events.addAction(self.actionJWatcher)
        self.menuExport_events.addAction(self.action_behavior_binary_table)
        self.menuObservations.addAction(self.actionNew_observation)
        self.menuObservations.addAction(self.actionOpen_observation)
        self.menuObservations.addAction(self.actionView_observation)
        self.menuObservations.addAction(self.actionEdit_observation_2)
        self.menuObservations.addAction(self.actionObservationsList)
        self.menuObservations.addAction(self.actionOpen_observation_2)
        self.menuObservations.addAction(self.actionEdit_observation)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.actionClose_observation)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.action_create_observations)
        self.menuObservations.addAction(self.actionRemove_observations)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.actionLoad_observations_file)
        self.menuObservations.addAction(self.actionExport_observations_list)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.actionAdd_event)
        self.menuObservations.addAction(self.actionEdit_selected_events)
        self.menuObservations.addAction(self.actionEdit_event_time)
        self.menuObservations.addAction(self.actionUndo)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.actionExplore_project)
        self.menuObservations.addAction(self.actionFind_events)
        self.menuObservations.addAction(self.actionFind_replace_events)
        self.menuObservations.addAction(self.actionCheckStateEvents)
        self.menuObservations.addAction(self.actionClose_unpaired_events)
        self.menuObservations.addAction(self.actionAdd_frame_indexes)
        self.menuObservations.addAction(self.actionSelect_observations)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.actionDelete_selected_events)
        self.menuObservations.addAction(self.actionDelete_all_events)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.menuExport_events.menuAction())
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.menuCreate_subtitles_2)
        self.menuObservations.addAction(self.actionExtract_events_from_media_files)
        self.menuObservations.addAction(self.actionExtract_frames_from_media_files)
        self.menuObservations.addSeparator()
        self.menuObservations.addAction(self.menuCreate_transitions_matrix.menuAction())
        self.menuPlot_events.addAction(self.actionPlot_events2)
        self.menuPlot_events.addAction(self.actionBehavior_bar_plot)
        self.menuInter_rater_reliability.addAction(self.actionCohen_s_kappa)
        self.menuSimilarities.addAction(self.actionNeedleman_Wunsch)
        self.menuAnalyze.addAction(self.actionTime_budget)
        self.menuAnalyze.addAction(self.actionTime_budget_by_behaviors_category)
        self.menuAnalyze.addAction(self.actionTime_budget_report)
        self.menuAnalyze.addAction(self.actionSynthetic_binned_time_budget)
        self.menuAnalyze.addAction(self.menuPlot_events.menuAction())
        self.menuAnalyze.addAction(self.menuInter_rater_reliability.menuAction())
        self.menuAnalyze.addAction(self.menuSimilarities.menuAction())
        self.menuAnalyze.addAction(self.action_cooccurence)
        self.menuAnalyze.addAction(self.action_advanced_event_filtering)
        self.menuAnalyze.addAction(self.action_latency)
        self.menuImage_overlay_on_video_2.addAction(self.actionAdd_image_overlay_on_video)
        self.menuImage_overlay_on_video_2.addAction(self.actionRemove_image_overlay)
        self.menuPlayback.addAction(self.actionPlay)
        self.menuPlayback.addAction(self.actionPrevious)
        self.menuPlayback.addAction(self.actionNext)
        self.menuPlayback.addAction(self.actionJumpForward)
        self.menuPlayback.addAction(self.actionJumpBackward)
        self.menuPlayback.addAction(self.actionJumpTo)
        self.menuPlayback.addAction(self.action_deinterlace)
        self.menuPlayback.addSeparator()
        self.menuPlayback.addAction(self.actionZoom_level)
        self.menuPlayback.addSeparator()
        self.menuPlayback.addAction(self.actionRotate_current_video)
        self.menuPlayback.addSeparator()
        self.menuPlayback.addAction(self.actionDisplay_subtitles)
        self.menuPlayback.addSeparator()
        self.menuPlayback.addAction(self.menuImage_overlay_on_video_2.menuAction())
        self.menuPlayback.addSeparator()
        self.menuPlayback.addAction(self.actionVideo_equalizer)
        self.menuTransitions_flow_diagram.addAction(self.actionCreate_transitions_flow_diagram)
        self.menuTransitions_flow_diagram.addAction(self.actionCreate_transitions_flow_diagram_2)
        self.menu_coding_map_creator.addAction(self.action_create_behaviors_coding_map)
        self.menu_coding_map_creator.addAction(self.action_create_modifiers_coding_map)
        self.menuMedia_file.addAction(self.actionMedia_file_information_2)
        self.menuMedia_file.addAction(self.actionRecode_resize_video)
        self.menuMedia_file.addAction(self.actionRotate_video)
        self.menuMedia_file.addAction(self.actionMerge_media_files)
        self.menuTools.addAction(self.menuMedia_file.menuAction())
        self.menuTools.addAction(self.action_block_dockwidgets)
        self.menuTools.addAction(self.actionPlot_events_in_real_time)
        self.menuTools.addAction(self.actionShow_spectrogram)
        self.menuTools.addAction(self.actionShow_the_sound_waveform)
        self.menuTools.addAction(self.actionShow_data_files)
        self.menuTools.addAction(self.action_geometric_measurements)
        self.menuTools.addAction(self.actionBehaviors_coding_map)
        self.menuTools.addAction(self.actionCoding_pad)
        self.menuTools.addAction(self.actionSubjects_pad)
        self.menuTools.addSeparator()
        self.menuTools.addAction(self.menu_coding_map_creator.menuAction())
        self.menuTools.addAction(self.menuTransitions_flow_diagram.menuAction())
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuObservations.menuAction())
        self.menubar.addAction(self.menuPlayback.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())
        self.menubar.addAction(self.menuAnalyze.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())
        self.toolBar.addAction(self.action_obs_list)
        self.toolBar.addAction(self.actionPlay)
        self.toolBar.addAction(self.actionReset)
        self.toolBar.addAction(self.actionJumpBackward)
        self.toolBar.addAction(self.actionJumpForward)
        self.toolBar.addAction(self.actionNormalSpeed)
        self.toolBar.addAction(self.actionFaster)
        self.toolBar.addAction(self.actionSlower)
        self.toolBar.addAction(self.actionPrevious)
        self.toolBar.addAction(self.actionNext)
        self.toolBar.addAction(self.actionSnapshot)
        self.toolBar.addAction(self.actionFrame_by_frame)
        self.toolBar.addAction(self.actionFrame_backward)
        self.toolBar.addAction(self.actionFrame_forward)
        self.toolBar.addAction(self.actionCloseObs)
        self.toolBar.addAction(self.actionPlot_events_in_real_time)
        self.toolBar.addAction(self.actionCurrent_Time_Budget)
        self.toolBar.addAction(self.actionPlot_current_observation)
        self.toolBar.addAction(self.actionPlot_current_time_budget)
        self.toolBar.addAction(self.action_geometric_measurements)
        self.toolBar.addAction(self.actionFind_in_current_obs)
        self.toolBar.addAction(self.actionExplore_project)
        self.toolBar.addAction(self.actionPreferences)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "BORIS"))
        self.lb_player_status.setText(_translate("MainWindow", "lb_player_status"))
        self.lb_current_media_time.setText(_translate("MainWindow", "lb_current_media_time"))
        self.lb_video_info.setText(_translate("MainWindow", "lb_video_info"))
        self.lb_zoom_level.setText(_translate("MainWindow", "lb_zoom_level"))
        self.lbFocalSubject.setText(_translate("MainWindow", "lbFocalSubject"))
        self.lbCurrentStates.setText(_translate("MainWindow", "lbCurrentStates"))
        self.pb_live_obs.setText(_translate("MainWindow", "PushButton"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.menuFile.setTitle(_translate("MainWindow", "Project"))
        self.menuRecent_projects.setTitle(_translate("MainWindow", "Recent projects"))
        self.menuImport_project.setTitle(_translate("MainWindow", "Import project from..."))
        self.menuMedia_file_Images_directories.setTitle(_translate("MainWindow", "External files"))
        self.menuSet_paths_relative_to_project_dir.setTitle(_translate("MainWindow", "Set paths relative to project dir"))
        self.menuRemove_paths.setTitle(_translate("MainWindow", "Remove paths"))
        self.menuObservations.setTitle(_translate("MainWindow", "Observations"))
        self.menuCreate_transitions_matrix.setTitle(_translate("MainWindow", "Create transitions matrix"))
        self.menuExport_events.setTitle(_translate("MainWindow", "Export events"))
        self.menuas_behavioural_sequences.setTitle(_translate("MainWindow", "as behavioural sequences"))
        self.menuAnalyze.setTitle(_translate("MainWindow", "Analysis"))
        self.menuPlot_events.setTitle(_translate("MainWindow", "Plot"))
        self.menuInter_rater_reliability.setTitle(_translate("MainWindow", "Inter-rater reliability"))
        self.menuSimilarities.setTitle(_translate("MainWindow", "Similarities"))
        self.menuPlayback.setTitle(_translate("MainWindow", "Playback"))
        self.menuImage_overlay_on_video_2.setTitle(_translate("MainWindow", "Image overlay on video"))
        self.menuTools.setTitle(_translate("MainWindow", "Tools"))
        self.menuTransitions_flow_diagram.setTitle(_translate("MainWindow", "Transitions flow diagram"))
        self.menu_coding_map_creator.setTitle(_translate("MainWindow", "Create a coding map"))
        self.menuMedia_file.setTitle(_translate("MainWindow", "Media file"))
        self.toolBar.setWindowTitle(_translate("MainWindow", "toolBar"))
        self.dwEthogram.setWindowTitle(_translate("MainWindow", "Ethogram"))
        self.twEthogram.setSortingEnabled(False)
        item = self.twEthogram.horizontalHeaderItem(0)
        item.setText(_translate("MainWindow", "Key"))
        item = self.twEthogram.horizontalHeaderItem(1)
        item.setText(_translate("MainWindow", "Code"))
        item = self.twEthogram.horizontalHeaderItem(2)
        item.setText(_translate("MainWindow", "Type"))
        item = self.twEthogram.horizontalHeaderItem(3)
        item.setText(_translate("MainWindow", "Description"))
        item = self.twEthogram.horizontalHeaderItem(4)
        item.setText(_translate("MainWindow", "Color"))
        item = self.twEthogram.horizontalHeaderItem(5)
        item.setText(_translate("MainWindow", "Category"))
        item = self.twEthogram.horizontalHeaderItem(6)
        item.setText(_translate("MainWindow", "Modifiers"))
        item = self.twEthogram.horizontalHeaderItem(7)
        item.setText(_translate("MainWindow", "Excluded"))
        self.dwEvents.setWindowTitle(_translate("MainWindow", "Events"))
        self.dwSubjects.setWindowTitle(_translate("MainWindow", "Subjects"))
        item = self.twSubjects.horizontalHeaderItem(0)
        item.setText(_translate("MainWindow", "Key"))
        item = self.twSubjects.horizontalHeaderItem(1)
        item.setText(_translate("MainWindow", "Name"))
        item = self.twSubjects.horizontalHeaderItem(2)
        item.setText(_translate("MainWindow", "Description"))
        item = self.twSubjects.horizontalHeaderItem(3)
        item.setText(_translate("MainWindow", "Current state(s)"))
        self.actionDocumentation.setText(_translate("MainWindow", "Documentation"))
        self.actionAbout.setText(_translate("MainWindow", "About"))
        self.actionQuit.setText(_translate("MainWindow", "Quit"))
        self.actionPause.setText(_translate("MainWindow", "Pause"))
        self.actionPause.setToolTip(_translate("MainWindow", "Pause"))
        self.actionPlay.setText(_translate("MainWindow", "Play"))
        self.actionOpen_video_file.setText(_translate("MainWindow", "Open media file"))
        self.actionReset.setText(_translate("MainWindow", "Reset"))
        self.actionFaster.setText(_translate("MainWindow", "Faster"))
        self.actionSlower.setText(_translate("MainWindow", "Slower"))
        self.actionJumpForward.setText(_translate("MainWindow", "Jump forward"))
        self.actionJumpForward.setShortcut(_translate("MainWindow", "Ctrl+F"))
        self.actionLoad_configuration.setText(_translate("MainWindow", "Load configuration"))
        self.actionDelete_selected_events.setText(_translate("MainWindow", "Delete selected events"))
        self.actionDelete_all_events.setText(_translate("MainWindow", "Delete all events"))
        self.actionSort_observations.setText(_translate("MainWindow", "Sort events"))
        self.actionLoad_observations_file.setText(_translate("MainWindow", "Import observations"))
        self.actionSelect_observations.setText(_translate("MainWindow", "Select events from interval"))
        self.actionConfigure_states_and_events.setText(_translate("MainWindow", "Configure states and events"))
        self.actionEdit_event.setText(_translate("MainWindow", "Edit event"))
        self.actionLoad_configuration_file.setText(_translate("MainWindow", "Load state and events configuration file"))
        self.actionMedia_file_information.setText(_translate("MainWindow", "Media file information"))
        self.actionStart_live_observation.setText(_translate("MainWindow", "Start observation without media file"))
        self.actionNew_project.setText(_translate("MainWindow", "New project"))
        self.actionTime_budget.setText(_translate("MainWindow", "Time budget"))
        self.actionSave_project.setText(_translate("MainWindow", "Save project"))
        self.actionSave_project.setShortcut(_translate("MainWindow", "Ctrl+S"))
        self.actionOpen_project.setText(_translate("MainWindow", "Open project"))
        self.actionSet_offset.setText(_translate("MainWindow", "Set time offset"))
        self.actionEdit_project.setText(_translate("MainWindow", "Edit project"))
        self.actionSave_project_as.setText(_translate("MainWindow", "Save project as ..."))
        self.actionPreferences.setText(_translate("MainWindow", "Preferences"))
        self.actionNew_observation.setText(_translate("MainWindow", "New observation"))
        self.actionNew_observation.setShortcut(_translate("MainWindow", "Ctrl+N"))
        self.actionSave_observation.setText(_translate("MainWindow", "Save current observation"))
        self.actionClose_observation.setText(_translate("MainWindow", "Close observation"))
        self.actionClose_observation.setShortcut(_translate("MainWindow", "Ctrl+Q"))
        self.actionEdit_current_observation.setText(_translate("MainWindow", "Edit current observation"))
        self.actionOpen_observation_2.setText(_translate("MainWindow", "Open observation"))
        self.actionAdd_event.setText(_translate("MainWindow", "Add event"))
        self.actionAdd_event.setShortcut(_translate("MainWindow", "Ctrl+A"))
        self.actionDeselectCurrentSubject.setText(_translate("MainWindow", "Deselect current subject"))
        self.actionNext.setText(_translate("MainWindow", "Next"))
        self.actionNext.setToolTip(_translate("MainWindow", "Next media file"))
        self.actionPrevious.setText(_translate("MainWindow", "Previous"))
        self.actionPrevious.setToolTip(_translate("MainWindow", "Previous media file"))
        self.actionJumpTo.setText(_translate("MainWindow", "Jump to specific time"))
        self.actionJumpBackward.setText(_translate("MainWindow", "Jump backward"))
        self.actionJumpBackward.setToolTip(_translate("MainWindow", "Jump backward"))
        self.actionJumpBackward.setShortcut(_translate("MainWindow", "Ctrl+B"))
        self.actionEdit_observation.setText(_translate("MainWindow", "Edit observation"))
        self.actionCheckUpdate.setText(_translate("MainWindow", "Check for updates and news"))
        self.actionClose_project.setText(_translate("MainWindow", "Close project"))
        self.actionObservationsList.setText(_translate("MainWindow", "Observations list"))
        self.actionObservationsList.setShortcut(_translate("MainWindow", "Ctrl+L"))
        self.actionNormalSpeed.setText(_translate("MainWindow", "Normal speed"))
        self.actionSnapshot.setText(_translate("MainWindow", "Snapshot"))
        self.actionFrame_by_frame.setText(_translate("MainWindow", "Frame by frame"))
        self.actionAggregatedEventsTabularFormat.setText(_translate("MainWindow", "Tab Separated Values (tsv)"))
        self.actionOpen_observation.setText(_translate("MainWindow", "Start observation"))
        self.actionOpen_observation.setShortcut(_translate("MainWindow", "Ctrl+O"))
        self.actionAaaa.setText(_translate("MainWindow", "aaaa"))
        self.menuCreate_subtitles_2.setText(_translate("MainWindow", "Create subtitles"))
        self.actionUser_guide.setText(_translate("MainWindow", "User guide"))
        self.actionEdit_observation_2.setText(_translate("MainWindow", "Edit observation"))
        self.actionEdit_observation_2.setShortcut(_translate("MainWindow", "Ctrl+E"))
        self.actionCheckStateEvents.setText(_translate("MainWindow", "Check state events"))
        self.actionRunEventOutside.setText(_translate("MainWindow", "Run external program with selected event(s)"))
        self.actionEdit_selected_events.setText(_translate("MainWindow", "Edit selected event(s)"))
        self.actionShow_spectrogram.setText(_translate("MainWindow", "Show the sound spectrogram"))
        self.actionExport_events_as_Praat_TextGrid.setText(_translate("MainWindow", "as Praat TextGrid"))
        self.actionExtract_events_from_media_files.setText(_translate("MainWindow", "Extract sequences from media files"))
        self.action_geometric_measurements.setText(_translate("MainWindow", "Geometric measurement"))
        self.actionFrame_forward.setText(_translate("MainWindow", "Frame forward"))
        self.actionFrame_backward.setText(_translate("MainWindow", "frame backward"))
        self.actionFilterBehaviors.setText(_translate("MainWindow", "Filter behaviors"))
        self.actionShowAllBehaviors.setText(_translate("MainWindow", "Show all behaviors"))
        self.actionShowAllBehaviors.setToolTip(_translate("MainWindow", "Show all behaviors"))
        self.actionExport_aggregated_events.setText(_translate("MainWindow", "Aggregated events"))
        self.actionCoding_pad.setText(_translate("MainWindow", "Coding pad"))
        self.actionTime_budget_by_behaviors_category.setText(_translate("MainWindow", "Time budget by behavioral category"))
        self.actionExport_events_as_SDIS_file.setText(_translate("MainWindow", "Export events as SDIS file"))
        self.actionRecode_resize_video.setText(_translate("MainWindow", "Re-encode/resize video"))
        self.actionMedia_file_information_2.setText(_translate("MainWindow", "Media file information"))
        self.actionCreate_transitions_flow_diagram.setText(_translate("MainWindow", "Create transitions DOT script"))
        self.actionCreate_transitions_flow_diagram_2.setText(_translate("MainWindow", "Create transitions flow diagram"))
        self.actionAll_transitions.setText(_translate("MainWindow", "Frequencies of transitions"))
        self.actionNumber_of_transitions.setText(_translate("MainWindow", "Number of transitions"))
        self.actionFrequencies_of_transitions_after_behaviors.setText(_translate("MainWindow", "Frequencies of transitions after behaviors"))
        self.actionFind_replace_events.setText(_translate("MainWindow", "Find/replace in events"))
        self.actionFind_events.setText(_translate("MainWindow", "Find in events"))
        self.actionFind_events.setToolTip(_translate("MainWindow", "Find"))
        self.actionHow_to_cite_BORIS.setText(_translate("MainWindow", "How to cite BORIS"))
        self.actionZoom1_1_1.setText(_translate("MainWindow", "1:1 Original"))
        self.actionZoom1_2_1.setText(_translate("MainWindow", "2:1 Double"))
        self.actionZoom1_1_2.setText(_translate("MainWindow", "1:2 Half"))
        self.actionZoom1_1_4.setText(_translate("MainWindow", "1:4 Quarter"))
        self.actionZoom1_fitwindow.setText(_translate("MainWindow", "Fit to window"))
        self.actionZoom2_fitwindow.setText(_translate("MainWindow", "Fit to window"))
        self.actionZoom2_1_4.setText(_translate("MainWindow", "1:4 Quarter"))
        self.actionZoom2_1_2.setText(_translate("MainWindow", "1:2 Half"))
        self.actionZoom2_1_1.setText(_translate("MainWindow", "1:1 Original"))
        self.actionZoom2_2_1.setText(_translate("MainWindow", "2:1 Double"))
        self.actionFilterSubjects.setText(_translate("MainWindow", "Filter subjects"))
        self.actionFilterSubjects.setToolTip(_translate("MainWindow", "Filter subjects"))
        self.actionShowAllSubjects.setText(_translate("MainWindow", "Show all subjects"))
        self.actionShowAllSubjects.setToolTip(_translate("MainWindow", "Show all subjects"))
        self.actionSend_project.setText(_translate("MainWindow", "Project server"))
        self.actionViewBehavior.setText(_translate("MainWindow", "View behavior"))
        self.actionViewBehavior.setToolTip(_translate("MainWindow", "View behavior"))
        self.actionBehaviors_coding_map.setText(_translate("MainWindow", "Behaviors coding map"))
        self.action_create_behaviors_coding_map.setText(_translate("MainWindow", "for behaviors"))
        self.action_create_modifiers_coding_map.setText(_translate("MainWindow", "for modifiers"))
        self.actionPlot_events1.setText(_translate("MainWindow", "Plot events (back compatibility)"))
        self.actionPlot_events2.setText(_translate("MainWindow", "Plot events"))
        self.actionCohen_s_kappa.setText(_translate("MainWindow", "Cohen\'s kappa (time-unit)"))
        self.actionTime_budget_report.setText(_translate("MainWindow", "Synthetic time budget"))
        self.actionCheckStateEventsSingleObs.setText(_translate("MainWindow", "Check state events"))
        self.actionSubjects_pad.setText(_translate("MainWindow", "Subjects pad"))
        self.actionShow_data_files.setText(_translate("MainWindow", "Show data files"))
        self.actionConverters_manager.setText(_translate("MainWindow", "Time converters manager"))
        self.actionView_observation.setText(_translate("MainWindow", "View observation"))
        self.actionRemove_path_from_media_files.setText(_translate("MainWindow", "Remove paths"))
        self.actionCheck_project.setText(_translate("MainWindow", "Check project integrity"))
        self.actionBehavior_bar_plot.setText(_translate("MainWindow", "Plot time budget"))
        self.action_obs_list.setText(_translate("MainWindow", "Obs list"))
        self.action_obs_list.setToolTip(_translate("MainWindow", "Observations list"))
        self.actionTest.setText(_translate("MainWindow", "test"))
        self.actionTest_stb2.setText(_translate("MainWindow", "test stb2"))
        self.actionClose_unpaired_events.setText(_translate("MainWindow", "Fix unpaired events"))
        self.actionClose_unpaired_events.setShortcut(_translate("MainWindow", "Ctrl+U"))
        self.actionRotate_video.setText(_translate("MainWindow", "Rotate video"))
        self.actionExport_observations_list.setText(_translate("MainWindow", "Export observations list"))
        self.actionExportEvents_2.setText(_translate("MainWindow", "Tabular events"))
        self.actionJWatcher.setText(_translate("MainWindow", "for analysis with JWatcher"))
        self.actionCloseObs.setText(_translate("MainWindow", "Close observation"))
        self.actionFilter_events.setText(_translate("MainWindow", "Filter events"))
        self.actionShow_all_events.setText(_translate("MainWindow", "Show all events"))
        self.actionShow_all_events.setToolTip(_translate("MainWindow", "Show all events"))
        self.actionEdit_event_time.setText(_translate("MainWindow", "Shift time of selected event(s)"))
        self.actionCopy_events.setText(_translate("MainWindow", "Copy events"))
        self.actionPaste_events.setText(_translate("MainWindow", "Paste events"))
        self.actionNoldus_Observer_template.setText(_translate("MainWindow", "Noldus The Observer XT (OTX, OTB, ODX)"))
        self.actionExplore_project.setText(_translate("MainWindow", "Explore project"))
        self.actionShow_the_sound_waveform.setText(_translate("MainWindow", "Show the sound waveform"))
        self.actionNeedleman_Wunsch.setText(_translate("MainWindow", "Needleman - Wunsch"))
        self.actionExtract_frames_from_media_files.setText(_translate("MainWindow", "Extract frames from media files"))
        self.action_behavior_binary_table.setText(_translate("MainWindow", "as behaviors binary table"))
        self.actionPlot_current_observation.setText(_translate("MainWindow", "Plot current observation"))
        self.actionFind_in_current_obs.setText(_translate("MainWindow", "Find in events"))
        self.action_advanced_event_filtering.setText(_translate("MainWindow", "Advanced event filtering"))
        self.action_block_dockwidgets.setText(_translate("MainWindow", "Lock dockwidgets"))
        self.actionseparated_subjects.setText(_translate("MainWindow", "separated subjects"))
        self.actiongrouped_subjects.setText(_translate("MainWindow", "grouped subjects"))
        self.actionCurrent_Time_Budget.setText(_translate("MainWindow", "Current Time Budget"))
        self.actionSynthetic_binned_time_budget.setText(_translate("MainWindow", "Synthetic time budget with time bin"))
        self.actionZoom_level.setText(_translate("MainWindow", "Zoom level"))
        self.actionDisplay_subtitles.setText(_translate("MainWindow", "Display subtitles"))
        self.actionAdd_image_overlay_on_video.setText(_translate("MainWindow", "Add"))
        self.actionRemove_image_overlay.setText(_translate("MainWindow", "Remove"))
        self.actionPlot_events_in_real_time.setText(_translate("MainWindow", "Plot events in real time"))
        self.actionVideo_equalizer.setText(_translate("MainWindow", "Video equalizer"))
        self.actionRemove_observations.setText(_translate("MainWindow", "Remove observations"))
        self.action_latency.setText(_translate("MainWindow", "Latency"))
        self.actionUndo.setText(_translate("MainWindow", "Undo"))
        self.actionaaa.setText(_translate("MainWindow", "aaa"))
        self.action_media_file_and_images_directories_relative_path.setText(_translate("MainWindow", "for media file and images directories"))
        self.action_data_files_relative_path.setText(_translate("MainWindow", "for data files"))
        self.action_remove_media_files_and_images_directories_path.setText(_translate("MainWindow", "for media file and images directories"))
        self.action_remove_data_files_path.setText(_translate("MainWindow", "for data files"))
        self.actionPlot_current_time_budget.setText(_translate("MainWindow", "Plot current time budget"))
        self.actionMerge_media_files.setText(_translate("MainWindow", "Merge media files"))
        self.actionExport_project.setText(_translate("MainWindow", "Export project"))
        self.actionConfigure_twEvents_columns.setText(_translate("MainWindow", "Configure columns"))
        self.actionRotate_current_video.setText(_translate("MainWindow", "Rotate video"))
        self.action_deinterlace.setText(_translate("MainWindow", "Deinterlace"))
        self.action_cooccurence.setText(_translate("MainWindow", "Co-occurence"))
        self.action_create_observations.setText(_translate("MainWindow", "Create observations"))
        self.actionConfigure_tvevents_columns.setText(_translate("MainWindow", "Configure columns"))
        self.actionAdd_frame_indexes.setText(_translate("MainWindow", "Add frame indexes"))
